from alr_spec.replacers.base import BaseReplacer


class ArchReplacer(BaseReplacer):
    ARCH_MAPPING = {
        "i686": "386",
        "x86_64": "amd64",
        "armv7h": "arm",
        "aarch64": "arm64",
    }

    def process(self):
        root_node = self.tree.root_node

        def find_replacements(node):
            if node.type == "variable_assignment":
                var_node = node.child_by_field_name("name")
                value_node = node.child_by_field_name("value")

                if var_node and value_node:
                    var_name = self._node_text(var_node)
                    if var_name == "architectures":
                        for item in value_node.children:
                            if item.type == "raw_string":
                                element_text = self._node_text(item)
                                if (
                                    element_text.startswith("'")
                                    and element_text.endswith("'")
                                ) or (
                                    element_text.startswith('"')
                                    and element_text.endswith('"')
                                ):
                                    quote_char = element_text[0]
                                    arch = element_text[1:-1]
                                else:
                                    arch = element_text

                                new_arch = self.ARCH_MAPPING.get(arch, arch)
                                new_element_text = (
                                    f"{quote_char}{new_arch}{quote_char}"
                                )

                                self.replaces.append(
                                    {
                                        "node": item,
                                        "content": new_element_text,
                                    }
                                )

            for child in node.children:
                find_replacements(child)

        find_replacements(root_node)
        return self._apply_replacements()



import typer

from alr_spec.core.update import update_package_logic

app = typer.Typer()


@app.command(help="Update package (version, release, checksums)")
def update_package(
    package_path: str = typer.Argument(
        ".", help="Путь к пакету (папке или alr.sh)"
    ),
    no_checksums: bool = typer.Option(
        False, "--no-checksums", help="Disable checksums update"
    ),
):
    try:
        update_package_logic(package_path, no_checksums)
    except Exception as e:
        typer.echo(f"Ошибка: {e}")
        raise typer.Exit(code=1)



import hashlib
import os
from pathlib import Path
from typing import List

import requests

from alr_spec.core.utils import run_bash_command
from alr_spec.replacers.base import BaseReplacer
from alr_spec.replacers.sources import Utils
from alr_spec.utils.from_pkgbuild import ShellReplacerProcessor


def _download_file(url: str, dest_folder: Path) -> Path:
    url_hash = hashlib.sha256(url.encode()).hexdigest()
    local_folder = dest_folder / url_hash
    local_folder.mkdir(parents=True, exist_ok=True)

    local_filename = local_folder / "file"
    if not local_filename.exists():
        print(f"Downloading file from {url}...")
        try:
            response = requests.get(url, stream=True, timeout=10000)
            response.raise_for_status()
            with open(local_filename, "wb") as f:
                for chunk in response.iter_content(chunk_size=8192):
                    f.write(chunk)
            print(f"Saved to {local_filename}")
        except requests.RequestException as e:
            raise RuntimeError(f"Download error: {e}")
    return local_filename


def _calculate_hash(file_path: Path, algorithm="sha256") -> str:
    hasher = hashlib.new(algorithm)
    with open(file_path, "rb") as f:
        while chunk := f.read(65536):
            hasher.update(chunk)
    return hasher.hexdigest()


def _get_replacer_class(checksums_field: str, values: List[str]):
    class ChecksumsReplacer(BaseReplacer):
        def process(self):
            def find_replacements(node):
                for child in node.children:
                    if child.type == "variable_assignment":
                        var_node, value_node = Utils.parse_variable_assignment(
                            child
                        )
                        if self._node_text(var_node) == checksums_field:
                            lst = "\n  ".join(values)
                            self.replaces.append(
                                {
                                    "node": value_node,
                                    "content": f"(\n  {lst}\n)",
                                }
                            )

            find_replacements(self.tree.root_node)
            return self._apply_replacements()

    return ChecksumsReplacer


def _update_checksums_for_field(
    script_path: Path, field: str, cache_dir: Path
):
    print(f"Updating checksums for {field}")
    checksums_field = field.replace("sources", "checksums")

    command = f"source {script_path} && printf '%s\\n' \"${{{field}[@]}}\""
    sources = run_bash_command(command).splitlines()

    checksums = []
    for url in sources:
        file_path = _download_file(url, cache_dir)
        hash_value = _calculate_hash(file_path)
        checksums.append(f"sha256:{hash_value}")

    with open(script_path, "rb") as f:
        pr = ShellReplacerProcessor()
        content = f.read()
        new_content = pr.process(
            content, [_get_replacer_class(checksums_field, checksums)]
        )

    with open(script_path, "wb") as f:
        f.write(new_content)


def update_all_checksums(package_path: str = ".", sources_field: str = ""):
    abs_path = Path(package_path).resolve()
    script_path = abs_path if abs_path.is_file() else abs_path / "alr.sh"

    if not script_path.exists():
        raise FileNotFoundError(f"{script_path} not found")

    command = f"source {script_path} && compgen -v | grep '^sources'"
    if sources_field:
        fields = [sources_field]
    else:
        fields = run_bash_command(command).splitlines()

    cache_dir = (
        Path(os.environ.get("XDG_CACHE_HOME", Path.home() / ".cache"))
        / "alr-spec"
        / "downloads"
    )

    for field in fields:
        _update_checksums_for_field(script_path, field, cache_dir)



import os

from alr_spec.core.replacer_factory import get_version_replacer_class
from alr_spec.core.utils import run_bash_command
from alr_spec.utils.from_pkgbuild import ShellReplacerProcessor


def _resolve_alr_path(package_path: str) -> str:
    if os.path.isfile(package_path):
        return os.path.abspath(package_path)

    candidate = os.path.join(package_path, "alr.sh")
    if not os.path.isfile(candidate):
        raise FileNotFoundError(
            f"'alr.sh' не найден в директории {package_path}"
        )
    return os.path.abspath(candidate)


def set_field_in_file(field: str, value: str, package_path: str = ".") -> None:
    file_path = _resolve_alr_path(package_path)

    with open(file_path, "rb") as f:
        pr = ShellReplacerProcessor()
        content = f.read()
        new_content = pr.process(
            content, [get_version_replacer_class(field, value)]
        )

    with open(file_path, "wb") as f:
        f.write(new_content)


def get_field_from_file(field: str, package_path: str = ".") -> str:
    file_path = _resolve_alr_path(package_path)
    command = f"source {file_path} && printf '%s\\n' \"${{{field}[@]}}\""
    return run_bash_command(command)



from typing import TYPE_CHECKING

from alr_spec.replacers.base import BaseReplacer
from alr_spec.replacers.sources import Utils

if TYPE_CHECKING:
    from tree_sitter import Node


def get_version_replacer_class(field, value):
    class VersionReplacer(BaseReplacer):
        def __init__(self, content, tree, ctx=None):
            super().__init__(content, tree, ctx)

        def process(self):
            root_node = self.tree.root_node

            def find_replacements(node: "Node"):
                for child in node.children:
                    if child.type == "variable_assignment":
                        res = Utils.parse_variable_assignment(child)
                        if res is not None:
                            var_node, value_node = res
                            var_name = self._node_text(var_node)

                            if var_name == field:
                                if value_node.type in [
                                    "string",
                                    "word",
                                    "number",
                                ]:
                                    self.replaces.append(
                                        {
                                            "node": value_node,
                                            "content": value,
                                        }
                                    )
                                else:
                                    print(value_node.type)

            find_replacements(root_node)
            return self._apply_replacements()

    return VersionReplacer



import re
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from tree_sitter import Node

from alr_spec.replacers.arch_replacer import ArchReplacer
from alr_spec.replacers.base import BaseReplacer


class StringValue(str):
    def __init__(self, node: "Node"):
        self.node = node

    def get_text_value(self) -> str:
        value = self.node.text.decode("utf-8")
        return value[1:-1]

    def get_quote(self) -> str:
        value = self.node.text.decode("utf-8")
        return value[0]

    def __repr__(self):
        return self.node.text.decode("utf-8")

    def __str__(self):
        return self.__repr__()


class Utils:
    def parse_variable_assignment(
        node: "Node",
    ) -> tuple["Node", "Node"] | None:
        var_node = node.child_by_field_name("name")
        value_node = node.child_by_field_name("value")

        if not (var_node and value_node):
            return None

        return (var_node, value_node)

    def get_string_values_from_array(node: "Node"):
        arr = []
        for item in node.children:
            if (
                item.type == "string"
                or item.type == "raw_string"
                or item.type == "concatenation"
            ):
                arr.append(StringValue(item))

        return arr


CHECKSUMS_REPLACEMENTS = {
    "b2sums": "blake2b-512",
    "sha512sums": "sha512",
    "sha384sums": "sha384",
    "sha256sums": "sha256",
    "sha224sums": "sha224",
    "sha1sums": "sha1",
    "md5sums": "md5",
}

SOURCE_PATTERN = r"^source(?:_(x86_64|i686|armv7h|aarch64))?$"
CHECKSUMS_PATTERN = (
    r"^(b2sums|sha512sums|sha384sums|sha256sums|sha224sums|sha1sums|md5sums)"
    r"(?:_(x86_64|i686|armv7h|aarch64))?$"
)


class SourcesReplacer(BaseReplacer):
    def process(self):
        root_node = self.tree.root_node

        self.local_files = []
        self.prepare_func_body = None

        self.nodes_to_remove = []

        sources = dict()
        checksums = dict()

        def execute(node: "Node"):
            if node.type == "function_definition":
                func_name = self._node_text(node.child_by_field_name("name"))
                if func_name != "prepare":
                    return
                self.prepare_func_body = node
                return

            if node.type == "variable_assignment":
                var_node = node.child_by_field_name("name")
                value_node = node.child_by_field_name("value")
                if not (var_node and value_node):
                    return

                var_name = self._node_text(var_node)

                re_match = re.match(SOURCE_PATTERN, var_name)
                if re_match:
                    self.nodes_to_remove.append(node)

                    arch = re_match.group(1)
                    sources[arch if arch else "-"] = (
                        Utils.get_string_values_from_array(value_node)
                    )

                re_match = re.match(CHECKSUMS_PATTERN, var_name)
                if re_match:
                    self.nodes_to_remove.append(node)

                    checksum = CHECKSUMS_REPLACEMENTS[re_match.group(1)]
                    arch = re_match.group(2)

                    checksums[arch if arch else "-"] = [
                        f"'{checksum}:{v.get_text_value()}'"
                        for v in Utils.get_string_values_from_array(value_node)
                    ]

        def traverse(node: "Node"):
            execute(node)
            for child in node.children:
                traverse(child)

        traverse(root_node)

        content = ""

        for node in self.nodes_to_remove:
            self.replaces.append({"node": node, "content": ""})

        for arch, files in sources.items():
            source_files = []
            checksums_str = []

            for i, file in enumerate(files):
                file_name = file.get_text_value()
                if "://" in file_name:
                    source_files.append(file)
                    checksums_str.append(checksums[arch][i])
                else:
                    self.local_files.append(file)

            content += self.source_to_str(arch, source_files) + "\n"
            content += self.checksums_to_str(arch, checksums_str) + "\n"

        if len(self.local_files) > 0:
            copy_commands = "\n    ".join(
                f'cp "${{scriptdir}}/{file.get_text_value()}" "${{srcdir}}"'
                for file in self.local_files
            )

            prepare_func_content = f"""
    {copy_commands}
"""
            if self.prepare_func_body is not None:
                text = self._node_text(self.prepare_func_body)
                closing_brace_index = text.rfind("}")
                text = (
                    text[:closing_brace_index]
                    + prepare_func_content
                    + text[closing_brace_index:]
                )
                self.replaces.append(
                    {
                        "node": self.prepare_func_body,
                        "content": text,
                    }
                )
            else:
                text = self._node_text(root_node)
                content += f"""
prepare() {{
{prepare_func_content}}}
"""

        self.appends.append(
            {
                "node": root_node,
                "content": content,
            }
        )

        return self._apply_replacements()

    def source_to_str(self, arch, files):
        return f"""
{f"sources_{ArchReplacer.ARCH_MAPPING[arch]}" if arch != '-' else "sources"}=(
  {'\n  '.join([s.__str__() for s in files])}
)"""

    def checksums_to_str(self, arch, files):
        var_name = (
            f"checksums_{ArchReplacer.ARCH_MAPPING[arch]}"
            if arch != "-"
            else "checksums"
        )

        return f"""
{var_name}=(
  {'\n  '.join([s.__str__() for s in files])}
)"""



import typer

from alr_spec.core.checksums import update_all_checksums

app = typer.Typer()


@app.command(help="Update checksums from sources")
def update_checksums(
    package_path: str = typer.Argument(".", help="Путь к alr.sh или папке"),
    sources_field: str = typer.Option(
        "",
        help="Конкретное имя переменной источника (например, sources_extra)",
    ),
):
    try:
        update_all_checksums(package_path, sources_field)
    except Exception as e:
        typer.echo(f"Ошибка: {e}")
        raise typer.Exit(code=1)



import re

from alr_spec.replacers.arch_replacer import ArchReplacer
from alr_spec.replacers.base import BaseReplacer


class SourcesReplacer(BaseReplacer):
    SOURCE_PATTERN = r"^source(?:_(x86_64|i686|armv7h|aarch64))?$"

    def process(self):
        root_node = self.tree.root_node

        def find_replacements(node):
            if node.type == "variable_name":
                var_name = self._node_text(node)
                re_match = re.match(self.SOURCE_PATTERN, var_name)

                if re_match:
                    arch = re_match.group(1)
                    self.replaces.append(
                        {
                            "node": node,
                            "content": (
                                f"sources_{ArchReplacer.ARCH_MAPPING[arch]}"
                                if arch
                                else "sources"
                            ),
                        }
                    )

            for child in node.children:
                find_replacements(child)

        find_replacements(root_node)
        return self._apply_replacements()



from typing import TYPE_CHECKING

from typing_extensions import List, TypedDict

if TYPE_CHECKING:
    from tree_sitter import Node, Tree


class Replaces(TypedDict):
    node: "Node"
    content: str


class Appends(TypedDict):
    node: "Node"
    content: str


class BaseReplacer:
    def __init__(self, content, tree: "Tree", ctx: dict | None = None):
        self.content = content
        self.tree = tree
        self.replaces: List[Replaces] = []
        self.appends: List[Appends] = []
        self.ctx = ctx

    def _node_text(self, node: "Node"):
        """Helper function to get the text of a node."""
        return self.content[node.start_byte : node.end_byte].decode("utf-8")

    def _apply_replacements(self):
        """Apply the replacements to the content and edit the tree."""
        new_content = bytearray(self.content)
        for replace_info in sorted(
            self.replaces,
            key=lambda x: (x["node"].start_byte, x["node"].end_byte),
            reverse=True,
        ):
            start, end = (
                replace_info["node"].start_byte,
                replace_info["node"].end_byte,
            )
            replacement = replace_info["content"].encode("utf-8")
            new_content[start:end] = replacement

            self.tree.edit(
                start_byte=start,
                old_end_byte=end,
                new_end_byte=start + len(replacement),
                start_point=replace_info["node"].start_point,
                old_end_point=replace_info["node"].end_point,
                new_end_point=(
                    replace_info["node"].start_point[0],
                    replace_info["node"].start_point[1] + len(replacement),
                ),
            )

        for append_info in sorted(
            self.appends,
            key=lambda x: x["node"].end_byte,
            reverse=True,
        ):
            insertion_point = append_info["node"].end_byte
            append_content = append_info["content"].encode("utf-8")
            new_content[insertion_point:insertion_point] = append_content

            self.tree.edit(
                start_byte=insertion_point,
                old_end_byte=insertion_point,
                new_end_byte=insertion_point + len(append_content),
                start_point=append_info["node"].end_point,
                old_end_point=append_info["node"].end_point,
                new_end_point=(
                    append_info["node"].end_point[0],
                    append_info["node"].end_point[1] + len(append_content),
                ),
            )

        return new_content

    def process(self):
        raise NotImplementedError("Subclasses should implement this method.")



import os
import subprocess
from abc import ABC
from pathlib import Path
from typing import Tuple

import requests

from alr_spec.core.checksums import update_all_checksums
from alr_spec.core.field_ops import get_field_from_file, set_field_in_file


class BaseUpdater(ABC):
    def update_check(self, package_path: Path) -> Tuple[str, str]:
        ...

        
    def update_run(self, package_path: Path, new_version: str) -> None:
        ...


class CommonUpdater(BaseUpdater):
    def update_check(self, package_path: Path):
        current_version = get_field_from_file("version", str(package_path))
        cmd = "nvchecker -c .nvchecker.toml --logger json | jq -r .version"
        result = subprocess.run(
            cmd, shell=True, capture_output=True, text=True, check=True
        )
        new_version = result.stdout.strip()
        return current_version, new_version

    def update_run(self, package_path: Path, new_version: str):
        set_field_in_file("version", new_version, str(package_path))
        set_field_in_file("release", "1", str(package_path))
        update_all_checksums(str(package_path), "")
        subprocess.run(["/bin/shfmt", "-l", "-w", "alr.sh"], check=True)
        
class SnapUpdater(CommonUpdater):
    def update_run(self, package_path: Path, new_version: str):
        snap_name = get_field_from_file("_snap_name", str(package_path))
        snap_channel = get_field_from_file("_snap_channel", str(package_path))
        
        headers = {"Snap-Device-Series": "16"}
        response = requests.get(f"https://api.snapcraft.io/v2/snaps/info/{snap_name}", headers=headers)
        response.raise_for_status()
        data = response.json()

        channel_map = data.get("channel-map", [])
        for entry in channel_map:
            channel = entry.get("channel", {})
            if channel.get("name") == snap_channel:
                version = entry.get("version")
                url = entry.get("download", {}).get("url")
                break
        else:
            raise ValueError(f"Channel {snap_channel} not found for snap {snap_name}")        
        
        set_field_in_file("_snap_url", url, str(package_path))
        set_field_in_file("version", new_version, str(package_path))
        set_field_in_file("release", "1", str(package_path))
        
        update_all_checksums(str(package_path), "")
        subprocess.run(["/bin/shfmt", "-l", "-w", "alr.sh"], check=True)


class Updater(BaseUpdater):
    def __init__(self, fallbackUpdater: BaseUpdater):
        self.updater = fallbackUpdater

    def update_check(self, package_path: Path):
        update_check_script = package_path / ".alr" / "update-check"
        if update_check_script.exists():
            print("Running .alr/update-check...")
            result = subprocess.run(
                [str(update_check_script)], shell=True, capture_output=True, text=True, check=True
            )
            res = result.stdout.split()
            return res[0], res[1]
        else:
            return self.updater.update_check(package_path)
    
    def update_run(self, package_path: Path, new_version: str):
        update_run_script = package_path / ".alr" / "update-run"
        if update_run_script.exists():
            print("Running .alr/update-run...")
            subprocess.run([str(update_run_script), new_version], check=True)
        else:
            return self.updater.update_run(package_path, new_version)


def update_package_logic(path_to_package: str, no_checksums: bool):
    package_path = Path(path_to_package).resolve()
    
    if not package_path.exists():
        raise FileNotFoundError(f"Path '{path_to_package}' does not exist.")
    
    os.chdir(package_path)
    
    preset = get_field_from_file("_alr_update_preset", package_path)
        
    if preset == "snap":
        fallbacUpdater = SnapUpdater()
    else:
        fallbacUpdater = CommonUpdater()        
        
    updater = Updater(fallbacUpdater)
    current_version, new_version = updater.update_check(package_path)
    if current_version == new_version:
        print("Nothing to do")
        return
    updater.update_run(package_path, new_version)



import sys
from typing import Annotated, Optional

import typer

from alr_spec.utils.empty_template import create_from_empty_template
from alr_spec.utils.from_pkgbuild import (
    PkgbuildDownloader,
    create_from_pkgbuild,
)

app = typer.Typer()


def process_empty_template(output_file: str):
    """Handles creation from an empty template."""
    typer.echo("Creating spec from an empty template...")
    create_from_empty_template(output_file)


def process_from_aur(package_name: str, output_file: str):
    """Handles creation from an AUR package."""
    typer.echo(
        f"Downloading PKGBUILD for package '{package_name}' from AUR..."
    )
    try:
        content = PkgbuildDownloader.download_and_extract(package_name)
        create_from_pkgbuild(content, output_file)
    except Exception as e:
        typer.echo(
            f"Error downloading PKGBUILD for '{package_name}': {e}", err=True
        )
        sys.exit(1)


def process_from_pkgbuild(file_path: str, output_file: str):
    """Handles creation from a local PKGBUILD file."""
    typer.echo(f"Reading PKGBUILD from local file '{file_path}'...")
    try:
        with open(file_path, "rb") as f:
            content = f.read()
        create_from_pkgbuild(content, output_file)
    except IOError as e:
        typer.echo(f"Error reading file '{file_path}': {e}", err=True)
        sys.exit(1)


@app.command(help="Create spec (empty, from PKGBUILD or AUR)")
def create(
    from_aur: Annotated[
        Optional[str], typer.Option(help="Package name to fetch from AUR")
    ] = None,
    from_pkgbuild: Annotated[
        Optional[str], typer.Option(help="Path to local PKGBUILD file")
    ] = None,
    empty_template: Annotated[
        Optional[bool], typer.Option(help="Create spec from an empty template")
    ] = None,
):
    """Main function to handle spec creation."""
    output_file = "alr.sh"

    if empty_template:
        process_empty_template(output_file)
    elif from_aur:
        process_from_aur(from_aur, output_file)
    elif from_pkgbuild:
        process_from_pkgbuild(from_pkgbuild, output_file)
    else:
        typer.echo(
            "No valid option provided. Use --help for usage details.", err=True
        )
        sys.exit(1)



from alr_spec.replacers.base import BaseReplacer


class SimpleReplacer(BaseReplacer):
    SIMPLE_REPLACEMENTS = {
        "pkgname": "name",
        "pkgver": "version",
        "pkgrel": "release",
        "pkgdesc": "desc",
        "url": "homepage",
        "arch": "architectures",
        "depends": "deps",
        "optdepends": "opt_deps",
    }

    def process(self):
        root_node = self.tree.root_node

        def find_replacements(node):
            if node.type == "variable_name":
                var_name = self._node_text(node)
                if var_name in self.SIMPLE_REPLACEMENTS:
                    self.replaces.append(
                        {
                            "node": node,
                            "content": self.SIMPLE_REPLACEMENTS[var_name],
                        }
                    )
            for child in node.children:
                find_replacements(child)

        find_replacements(root_node)
        return self._apply_replacements()



import os
import subprocess

import typer

app = typer.Typer()

IMAGE = "ghcr.io/aides-infra/buildbot-worker-builder-sisyphus-x86_64:0.0.7"
BUILD_COMMAND = (
    "cd /app && /bin/aides-update-cache && /bin/alr fix &&"
    + "/bin/alr -i=false build -s $(pwd)/alr.sh"
)


@app.command(help="Build package from spec in isolated environment")
def build(
    no_cache: bool = typer.Option(False, "--no-cache", help="Disable cache")
):
    command = f"source {os.getcwd()}/alr.sh && echo $name"

    result = subprocess.run(
        command, shell=True, capture_output=True, text=True
    )

    if result.returncode != 0:
        print(result)
        exit(-1)

    package_name = result.stdout.strip()

    cache_dir = os.path.join(
        os.environ.get("XDG_CACHE_HOME", os.path.expanduser("~/.cache")),
        "alr-spec",
        "builds",
        package_name,
    )

    try:
        command = [
            "docker",
            "run",
            "-it",
            "--rm",
            "--device",
            "/dev/fuse",
            "--cap-add=SYS_ADMIN",
            "-v",
            f"{os.getcwd()}:/app",
        ]

        if not no_cache:
            alr_cache = os.path.join(cache_dir, "alr")
            os.makedirs(alr_cache, exist_ok=True)
            command.extend(["-v", f"{alr_cache}:/var/cache/alr"])

            apt_archives = os.path.join(cache_dir, "apt_archives")
            os.makedirs(apt_archives, exist_ok=True)
            os.makedirs(os.path.join(apt_archives, "partial"), exist_ok=True)
            command.extend(["-v", f"{apt_archives}:/var/cache/apt/archives"])

        command.extend([IMAGE, "/bin/sh", "-c", BUILD_COMMAND])

        print(" ".join(command))

        subprocess.run(command, check=True)
    except subprocess.CalledProcessError as e:
        typer.echo(f"Error while running docker: {e}")
    except FileNotFoundError:
        typer.echo("Docker is not installed or not in PATH.")



import typer

from alr_spec.core.field_ops import get_field_from_file, set_field_in_file

app = typer.Typer()


@app.command(help="Set field in alr.sh")
def set_field(
    field: str,
    value: str,
    package_path: str = typer.Option(
        ".", help="Путь к alr.sh или к папке, где он находится"
    ),
):
    set_field_in_file(field, value, package_path)


@app.command(help="Get field from alr.sh")
def get_field(
    field: str,
    package_path: str = typer.Option(
        ".", help="Путь к alr.sh или к папке, где он находится"
    ),
):
    result = get_field_from_file(field, package_path)
    print(result)



import os
import shutil
import sys
import tempfile

import git
import tree_sitter_bash as tsbash
from tree_sitter import Language, Parser

from alr_spec.replacers.arch_replacer import ArchReplacer
from alr_spec.replacers.simple_replacer import SimpleReplacer
from alr_spec.replacers.sources import SourcesReplacer

HEADER = """#
# WARNING: Automatically converted from PKGBUILD and may contain errors
#
"""


class PkgbuildDownloader:
    """Handles downloading PKGBUILD and associated files."""

    @staticmethod
    def download_and_extract(pkgname: str) -> bytes:
        aur_url = f"https://aur.archlinux.org/{pkgname}.git"

        with tempfile.TemporaryDirectory() as tmpdirname:
            try:
                print(f"Cloning repository for {pkgname}...")
                git.Repo.clone_from(aur_url, tmpdirname)
                print(f"Files for {pkgname} downloaded to {tmpdirname}")

                PkgbuildDownloader._copy_files(tmpdirname)

                with open(os.path.join(tmpdirname, "PKGBUILD"), "rb") as f:
                    return f.read()

            except Exception as e:
                print(f"Error downloading repository: {e}", file=sys.stderr)
                sys.exit(1)

    @staticmethod
    def _copy_files(tmpdirname: str):
        """Copies all files to the current directory."""
        for root, dirs, files in os.walk(tmpdirname):
            dirs[:] = [d for d in dirs if d not in [".git"]]
            files = [f for f in files if f not in ["PKGBUILD", ".SRCINFO"]]

            for file in files:
                file_path = os.path.join(root, file)
                relative_path = os.path.relpath(file_path, tmpdirname)
                destination_path = os.path.join(os.getcwd(), relative_path)

                os.makedirs(os.path.dirname(destination_path), exist_ok=True)
                shutil.copy(file_path, destination_path)


class ShellReplacerProcessor:
    """Processes PKGBUILD files with replacers."""

    def __init__(self):
        self.parser = self._initialize_parser()

    @staticmethod
    def _initialize_parser() -> Parser:
        bash_language = Language(tsbash.language())
        return Parser(bash_language)

    def process(self, content: bytes, replacers: list) -> bytes:
        tree = self.parser.parse(content)

        for replacer_class in replacers:
            replacer = replacer_class(content, tree)
            content = replacer.process()
            tree = self.parser.parse(content, tree)

        return content


def create_from_pkgbuild(content: bytes, output_file: str):
    """Creates a new spec file from a PKGBUILD."""
    processor = ShellReplacerProcessor()

    replacers = [
        SimpleReplacer,
        ArchReplacer,
        SourcesReplacer,
    ]

    try:
        new_content = processor.process(content, replacers)
        new_content = bytes(HEADER, encoding="utf-8") + new_content

        with open(output_file, "wb") as f:
            f.write(new_content)

        print(f"File successfully written to {output_file}.")
    except IOError as e:
        print(f"Error writing file: {e}", file=sys.stderr)
        sys.exit(1)



import typer

from alr_spec.commands.build import app as build_app
from alr_spec.commands.checksums import app as checksums_app
from alr_spec.commands.create import app as create_app
from alr_spec.commands.field import app as field_app
from alr_spec.commands.update import app as update_package_app

app = typer.Typer()

app.add_typer(create_app)
app.add_typer(build_app)
app.add_typer(checksums_app)
app.add_typer(field_app)
app.add_typer(update_package_app)


def main():
    app()


if __name__ == "__main__":
    main()



import sys

file = """name=example
version=1.0.0
release=1
epoch=0
desc=Description
homepage=https://example.com
maintainer="Name Surname <email@example.com>"
# or 'all' for noarch
architectures=('amd64')
license=('custom:proprietary')
provides=()
conflicts=()
deps=()
build_deps=()
opt_deps=()
auto_prov=0
auto_req=0
replaces=()
sources=()
checksums=()
backup=()
scripts=(
  ['preinstall']='preinstall.sh'
  ['postinstall']='postinstall.sh'
  ['preremove']='preremove.sh'
  ['postremove']='postremove.sh'
  ['preupgrade']='preupgrade.sh'
  ['postupgrade']='postupgrade.sh'
  ['pretrans']='pretrans.sh'
  ['posttrans']='posttrans.sh'
)

prepare() {
    echo "PREPARE"
}
build() {
    echo "BUILD"
}
package() {
    echo "PACKAGE"
}
files() {
    echo ./usr/bin/example
}
"""


def create_from_empty_template(output_file):
    try:
        with open(output_file, "w") as f:
            f.write(file)
        print(f"Файл успешно записан в {output_file}.")
    except IOError as e:
        print(f"Ошибка при записи файла: {e}")
        sys.exit(1)



import subprocess


def run_bash_command(command):
    result = subprocess.run(
        ["/bin/bash", "-c", command],
        shell=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    if result.returncode != 0:
        raise RuntimeError(f"Ошибка при выполнении команды: {result.stderr}")
    return result.stdout.strip()
